/**
 * 
 */
package com.github.ansell.oas.tsv.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.Reader;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.openrdf.model.Model;

import com.github.ansell.oas.tsv.OasTsvUtility;

/**
 * Test for {@link OasTsvUtility}.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class OasTsvUtilityTest
{
    @Rule
    public TemporaryFolder junitTempFolder = new TemporaryFolder();
    
    private File testDataFolder;
    
    private File tempFile;
    
    private void assertModel(final Model parseTsv)
    {
        Assert.assertFalse(parseTsv.isEmpty());
        Assert.assertEquals(4, parseTsv.subjects().size());
        Assert.assertEquals(6, parseTsv.predicates().size());
        Assert.assertEquals(9, parseTsv.objects().size());
    }
    
    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception
    {
        this.testDataFolder = this.junitTempFolder.newFolder(this.getClass().getName());
        this.tempFile = File.createTempFile("OasTsvUtilityTest-oastsvexample-", ".tsv", this.testDataFolder);
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/tsv/oastsvexample.tsv");
        
        Assert.assertNotNull("Could not find test resource", testInputStream);
        
        Files.copy(testInputStream, this.tempFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
    }
    
    /**
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception
    {
        this.tempFile.delete();
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.tsv.OasTsvUtility#parseTsv(java.io.InputStream, boolean)}.
     */
    @Test
    public final void testParseTsvInputStreamBoolean() throws Exception
    {
        try (final InputStream testInputStream = new FileInputStream(this.tempFile))
        {
            final Model parseTsv = OasTsvUtility.parseTsv(testInputStream, true);
            this.assertModel(parseTsv);
        }
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.tsv.OasTsvUtility#parseTsv(java.io.Reader, boolean)}.
     */
    @Test
    public final void testParseTsvReaderBoolean() throws Exception
    {
        try (final Reader testReader = new FileReader(this.tempFile))
        {
            final Model parseTsv = OasTsvUtility.parseTsv(testReader, true);
            this.assertModel(parseTsv);
        }
    }
    
    /**
     * Test method for {@link com.github.ansell.oas.tsv.OasTsvUtility#parseTsv(java.lang.String)}.
     * 
     * @throws Exception
     */
    @Test
    public final void testParseTsvString() throws Exception
    {
        final Model parseTsv = OasTsvUtility.parseTsv(this.tempFile.getAbsolutePath());
        this.assertModel(parseTsv);
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.tsv.OasTsvUtility#parseTsv(java.lang.String, boolean)}.
     */
    @Test
    public final void testParseTsvStringBoolean() throws Exception
    {
        final Model parseTsv = OasTsvUtility.parseTsv(this.tempFile.getAbsolutePath(), true);
        this.assertModel(parseTsv);
    }
}
