/**
 * 
 */
package com.github.ansell.oas.utils.test;

import java.util.HashSet;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;

import com.github.ansell.oas.utils.DateUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class DateUtilsTest
{
    /**
     * Test method for {@link com.github.ansell.oas.utils.DateUtils#getCurrentDate()}.
     */
    @Test
    public final void testGetCurrentDate()
    {
        Assert.assertNotNull("Date string was null", DateUtils.getCurrentDate());
        Assert.assertNotSame("Date string was empty", "", DateUtils.getCurrentDate().trim());
    }
    
    /**
     * Test method for {@link com.github.ansell.oas.utils.DateUtils#getCurrentDateLiteral()}.
     */
    @Test
    public final void testGetCurrentDateLiteral()
    {
        Assert.assertNotNull("Date literal was null", DateUtils.getCurrentDateLiteral());
        Assert.assertNotNull("Date literal date type was null", DateUtils.getCurrentDateLiteral().getDatatype());
        Assert.assertNotNull("Date literal calendar value was null", DateUtils.getCurrentDateLiteral().calendarValue());
    }
    
    /**
     * Stress test to verify that duplicates are very rare.
     * 
     * Test method for
     * {@link com.github.ansell.oas.utils.DateUtils#getNewUniqueUriByDate(java.lang.String, java.lang.String)}
     * .
     */
    @Test
    public final void testGetNewUniqueUriByDateMultiple()
    {
        final Set<URI> results = new HashSet<URI>();
        for(int i = 0; i < 1000000; i++)
        {
            final URI nextResult = DateUtils.getNewUniqueUriByDate("http://example.org/test/date/", "abc123");
            Assert.assertNotNull("Date was null", nextResult);
            // check that the URI was not in the results at this stage
            Assert.assertFalse("Found a duplicate at iteration: " + i, results.contains(nextResult));
            results.add(nextResult);
            // verify that there are exactly the right number of statements in the results at this
            // stage
            Assert.assertEquals(i + 1, results.size());
        }
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.DateUtils#getNewUniqueUriByDate(java.lang.String, java.lang.String)}
     * .
     */
    @Test
    public final void testGetNewUniqueUriByDateSingle()
    {
        Assert.assertNotNull("Date was null",
                DateUtils.getNewUniqueUriByDate("http://example.org/test/date/", "abc123"));
    }
    
}
