/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import java.io.StringReader;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.Model;
import org.openrdf.rio.RDFFormat;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.RestletUtilMediaType;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class AuditLogResourceImplTest extends AbstractResourceImplTest
{
    
    @Test
    public void testGetAuditLogAuthenticatedAdminHtml() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        
        this.assertFreemarker(result);
        
        Assert.assertTrue(result.contains("List of audited activities"));
        Assert.assertTrue(result.contains("auditList"));
        Assert.assertFalse(result.contains("There are no currently available audited activities in the audit log"));
    }
    
    @Test
    public void testGetAuditLogAuthenticatedAdminRdfJson() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        RestletUtilMediaType.APPLICATION_RDF_JSON, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        
        this.assertFreemarker(result);
        
        this.log.error(result);
        
        final Model model = this.assertRdf(new StringReader(result), RDFFormat.RDFJSON, 15);
    }
    
    @Test
    public void testGetAuditLogAuthenticatedAdminRdfXml() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        
        this.assertFreemarker(result);
        final Model model = this.assertRdf(new StringReader(result), RDFFormat.RDFXML, 15);
    }
    
    @Test
    public void testGetAuditLogAuthenticatedAdminTurtle() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.APPLICATION_RDF_TURTLE, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        
        this.assertFreemarker(result);
        final Model model = this.assertRdf(new StringReader(result), RDFFormat.TURTLE, 15);
    }
    
    @Test
    public void testGetAuditLogAuthenticatedNonAdminHtml() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null, MediaType.TEXT_HTML,
                    Status.SUCCESS_OK, this.testNoAdminPrivileges);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogAuthenticatedNonAdminRdfJson() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                    RestletUtilMediaType.APPLICATION_RDF_JSON, Status.SUCCESS_OK, this.testNoAdminPrivileges);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogAuthenticatedNonAdminRdfXml() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                    MediaType.APPLICATION_RDF_XML, Status.SUCCESS_OK, this.testNoAdminPrivileges);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogAuthenticatedNonAdminTurtle() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                    MediaType.APPLICATION_RDF_TURTLE, Status.SUCCESS_OK, this.testNoAdminPrivileges);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogUnauthenticatedHtml() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        try
        {
            creationClientResource.get(MediaType.TEXT_HTML);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogUnauthenticatedRdfJson() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        try
        {
            creationClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogUnauthenticatedRdfXml() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        try
        {
            creationClientResource.get(MediaType.APPLICATION_RDF_XML);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    @Test
    public void testGetAuditLogUnauthenticatedTurtle() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_AUDITLOG_PATH,
                        OasProps.DEF_AUDITLOG_PATH)));
        try
        {
            creationClientResource.get(MediaType.APPLICATION_RDF_TURTLE);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
}
