/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.openrdf.model.URI;
import org.openrdf.model.impl.ValueFactoryImpl;
import org.openrdf.rio.RDFFormat;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class OntologyBrowserResourceImplTest extends AbstractOntologyManagerTest
{
    private String ontologyUri;
    private String ontologyVersionUri;
    private String baseUri;
    
    /*
     * (non-Javadoc)
     * 
     * @see com.github.ansell.oas.webservice.impl.test.AbstractResourceImplTest#setUp()
     */
    @Override
    @Before
    public void setUp() throws Exception
    {
        super.setUp();
        
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        this.ontologyUri = "http://purl.org/obo/owl/test";
        this.ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        this.baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", this.ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", this.ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", this.baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        // final String rdfDocument =
        // TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri,
        // ontologyVersionUri);
        //
        // final Representation input =
        // new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML,
        // Language.DEFAULT,
        // CharacterSet.UTF_8);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        final String stringInput = IOUtils.toString(inputStream);
        
        // AbstractResourceImplTest.LOG.trace("input=");
        // AbstractResourceImplTest.LOG.trace(stringInput);
        
        // final Representation input = new InputRepresentation(inputStream,
        // MediaType.APPLICATION_RDF_XML);
        
        final Representation input = new StringRepresentation(stringInput, MediaType.APPLICATION_RDF_XML);
        
        // final Representation results = creationClientResource.put(input);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), this.baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
        }
        
        this.checkOntologyManager(MediaType.APPLICATION_RDF_XML, RDFFormat.RDFXML, 1, 1,
                Arrays.asList(ValueFactoryImpl.getInstance().createURI(this.ontologyUri)),
                Arrays.asList(ValueFactoryImpl.getInstance().createURI(this.ontologyVersionUri)));
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.github.ansell.oas.webservice.impl.test.AbstractResourceImplTest#tearDown()
     */
    @Override
    @After
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    @Test
    public void testOntologyBrowserDirectHtml() throws Exception
    {
        final ClientResource ontologyBrowserClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_BROWSER_PATH,
                        OasProps.DEF_ONTOLOGY_BROWSER_PATH)));
        
        ontologyBrowserClientResource.getReference().addQueryParameter("ontologyUri", this.ontologyUri);
        ontologyBrowserClientResource.getReference().addQueryParameter("ontologyVersionUri", this.ontologyVersionUri);
        ontologyBrowserClientResource.getReference().addQueryParameter("directChildrenOnly", "true");
        
        final Representation representation = ontologyBrowserClientResource.get(MediaType.TEXT_HTML);
        
        Assert.assertNotNull(representation);
        Assert.assertEquals(Status.SUCCESS_OK.getCode(), ontologyBrowserClientResource.getStatus().getCode());
        
        // this.log.trace("result={}", representation.getText());
        final String resultsText = representation.getText();
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-testwebsite test Ajaxified Ontology Browser template"));
        
    }
    
    @Test
    public void testOntologyBrowserDirectJson() throws Exception
    {
        final ClientResource ontologyBrowserClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_BROWSER_PATH,
                        OasProps.DEF_ONTOLOGY_BROWSER_PATH)));
        
        ontologyBrowserClientResource.getReference().addQueryParameter("ontologyUri", this.ontologyUri);
        ontologyBrowserClientResource.getReference().addQueryParameter("ontologyVersionUri", this.ontologyVersionUri);
        ontologyBrowserClientResource.getReference().addQueryParameter("directChildrenOnly", "true");
        
        final Representation representation = ontologyBrowserClientResource.get(MediaType.APPLICATION_JSON);
        
        Assert.assertNotNull(representation);
        Assert.assertEquals(Status.SUCCESS_OK.getCode(), ontologyBrowserClientResource.getStatus().getCode());
        
        // this.log.trace("result={}", representation.getText());
    }
    
    @Test
    public void testOntologyBrowserDirectRdfXml() throws Exception
    {
        final ClientResource ontologyBrowserClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_BROWSER_PATH,
                        OasProps.DEF_ONTOLOGY_BROWSER_PATH)));
        
        ontologyBrowserClientResource.getReference().addQueryParameter("ontologyUri", this.ontologyUri);
        ontologyBrowserClientResource.getReference().addQueryParameter("ontologyVersionUri", this.ontologyVersionUri);
        ontologyBrowserClientResource.getReference().addQueryParameter("directChildrenOnly", "true");
        
        final Representation representation = ontologyBrowserClientResource.get(MediaType.APPLICATION_RDF_XML);
        
        Assert.assertNotNull(representation);
        Assert.assertEquals(Status.SUCCESS_OK.getCode(), ontologyBrowserClientResource.getStatus().getCode());
        
        // this.log.trace("result={}", representation.getText());
        
    }
    
    @Test
    public void testOntologyBrowserNoParameters() throws Exception
    {
        final ClientResource ontologyBrowserClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_BROWSER_PATH,
                        OasProps.DEF_ONTOLOGY_BROWSER_PATH)));
        
        try
        {
            ontologyBrowserClientResource.get();
            Assert.fail("Expected exception when querying the ontology browser without an ontology URI");
        }
        catch(final ResourceException e)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST.getCode(), ontologyBrowserClientResource.getStatus()
                    .getCode());
        }
    }
}
