/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.openrdf.model.impl.ValueFactoryImpl;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.helpers.StatementCollector;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Preference;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.RestletUtilMediaType;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class OntologyManagerResourceImplTest extends AbstractOntologyManagerTest
{
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri but not
     * deleteOldVersions.
     * 
     * @throws Exception
     */
    @Test
    public void testOntologyCreationMediumSizeRdfXml2RdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        // final String rdfDocument =
        // TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri,
        // ontologyVersionUri);
        //
        // final Representation input =
        // new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML,
        // Language.DEFAULT,
        // CharacterSet.UTF_8);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        final String stringInput = IOUtils.toString(inputStream);
        
        // this.log.trace("input=");
        // this.log.trace(stringInput);
        
        // final Representation input = new InputRepresentation(inputStream,
        // MediaType.APPLICATION_RDF_XML);
        
        final Representation input = new StringRepresentation(stringInput, MediaType.APPLICATION_RDF_XML);
        
        // final Representation results = creationClientResource.put(input);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
        }
        
        this.checkOntologyManager(MediaType.APPLICATION_RDF_XML, RDFFormat.RDFXML, 1, 1,
                Arrays.asList(ValueFactoryImpl.getInstance().createURI(ontologyUri)),
                Arrays.asList(ValueFactoryImpl.getInstance().createURI(ontologyVersionUri)));
    }
    
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri but not
     * deleteOldVersions.
     * 
     * @throws Exception
     */
    @Test
    public void testOntologyCreationRdfXml2RdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        // final Representation results = creationClientResource.put(input,
        // MediaType.APPLICATION_RDF_XML);
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
        }
        
        this.checkOntologyManager(MediaType.APPLICATION_RDF_XML, RDFFormat.RDFXML, 1, 1,
                Arrays.asList(ValueFactoryImpl.getInstance().createURI(ontologyUri)),
                Arrays.asList(ValueFactoryImpl.getInstance().createURI(ontologyVersionUri)));
        
        // Get the resource and dump it to the console
        final ClientResource getClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation getResults = getClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), getResults.getMediaType().getName());
        
        final String result = getResults.getText();
        this.assertFreemarker(result);
        
        this.log.debug(result);
        
        Assert.assertTrue(result.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(result.contains("<table class=\"table table-striped table-hover\" id=\"ontologyList\">"));
        
        Assert.assertTrue(result.contains("<a class=\"ontology\""));
    }
    
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri with deleteOldVersions
     * set to false.
     * 
     * @throws Exception
     * @throws RDFHandlerException
     * @throws RDFParseException
     */
    @Test
    public void testOntologyCreationRdfXml2RdfXmlDeleteOldVersionsFalse() throws Exception
    {
        final String ontologyUri = "http://purl.org/obo/owl/test";
        
        final Set<URI> expectedUris = new HashSet<URI>();
        
        for(int count = 1; count <= 10; count++)
        {
            // Get the resource and dump it to the console
            final ClientResource nextCreationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String newOntologyVersionUri = "http://purl.org/obo/owl/test/version:" + count;
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            expectedUris.add(ValueFactoryImpl.getInstance().createURI(newOntologyVersionUri));
            
            nextCreationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            nextCreationClientResource.getReference().addQueryParameter("ontologyVersionUri", newOntologyVersionUri);
            nextCreationClientResource.getReference().addQueryParameter("deleteOldVersions", "false");
            nextCreationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String nextTestRdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, newOntologyVersionUri);
            
            final Representation nextInput =
                    new AppendableRepresentation(nextTestRdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            final Representation nextResults =
                    RestletTestUtils.doTestAuthenticatedRequest(nextCreationClientResource, Method.PUT, nextInput,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), nextResults.getMediaType().getName());
            
            // Test that after each execution with deleteOldVersions=false, that the number of
            // versions displayed in the RDF/XML version of the ontology manager increases to match
            // the number of known versions
            this.checkOntologyManager(MediaType.APPLICATION_RDF_XML, RDFFormat.RDFXML, 1, count,
                    Arrays.asList(ValueFactoryImpl.getInstance().createURI(ontologyUri)), expectedUris);
        }
        
        Assert.assertEquals(10, expectedUris.size());
        
        // Get the resource and dump it to the console
        final ClientResource getClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation getResults = getClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), getResults.getMediaType().getName());
        
        final String resultString = IOUtils.toString(getResults.getStream());
        
        this.log.debug(resultString);
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString
                .contains("<table class=\"table table-striped table-hover\" id=\"ontologyList\">"));
        
        Assert.assertTrue(resultString.contains("<a class=\"ontology\" "));
        
        for(final URI nextExpectedUri : expectedUris)
        {
            Assert.assertTrue(resultString.contains("<input type=\"hidden\" name=\"ontologyVersionUri\" value=\""
                    + nextExpectedUri.stringValue() + "\"/>"));
        }
    }
    
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri with deleteOldVersions
     * set to false.
     * 
     * @throws Exception
     * @throws RDFHandlerException
     * @throws RDFParseException
     */
    @Test
    public void testOntologyCreationRdfXml2RdfXmlDeleteOldVersionsTrue() throws Exception
    {
        final String ontologyUri = "http://purl.org/obo/owl/test";
        
        for(int count = 1; count <= 10; count++)
        {
            // Get the resource and dump it to the console
            final ClientResource nextCreationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String newOntologyVersionUri = "http://purl.org/obo/owl/test/version:" + count;
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            nextCreationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            nextCreationClientResource.getReference().addQueryParameter("ontologyVersionUri", newOntologyVersionUri);
            nextCreationClientResource.getReference().addQueryParameter("deleteOldVersions", "true");
            nextCreationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String nextTestRdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, newOntologyVersionUri);
            
            final Representation nextInput =
                    new AppendableRepresentation(nextTestRdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            final Representation nextResults =
                    RestletTestUtils.doTestAuthenticatedRequest(nextCreationClientResource, Method.PUT, nextInput,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), nextResults.getMediaType().getName());
            
            // Test that after each execution with deleteOldVersions that there is only a single
            // instance of the ontology in the ontology manager web interface
            this.checkOntologyManager(MediaType.APPLICATION_RDF_XML, RDFFormat.RDFXML, 1, 1,
                    Arrays.asList(ValueFactoryImpl.getInstance().createURI(ontologyUri)),
                    Arrays.asList(ValueFactoryImpl.getInstance().createURI(newOntologyVersionUri)));
        }
        
        // Get the resource and dump it to the console
        final ClientResource getClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation getResults = getClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), getResults.getMediaType().getName());
        
        final String resultString = IOUtils.toString(getResults.getStream());
        
        this.log.debug(resultString);
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString
                .contains("<table class=\"table table-striped table-hover\" id=\"ontologyList\">"));
        
        Assert.assertTrue(resultString.contains("<a class=\"ontology\""));
        
        Assert.assertTrue(resultString
                .contains("<input type=\"hidden\" name=\"ontologyVersionUri\" value=\"http://purl.org/obo/owl/test/version:10\"/>"));
        
        for(int i = 1; i < 10; i++)
        {
            Assert.assertFalse(resultString
                    .contains("<input type=\"hidden\" name=\"ontologyVersionUri\" value=\"http://purl.org/obo/owl/test/version:"
                            + i + "\"/>"));
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyDeleteAllVersionsRdfXml2RdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        // final Representation results = creationClientResource.put(input,
        // MediaType.APPLICATION_RDF_XML);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
            
            final Ontology nextOntology = nextOntologies.get(0);
            
            Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
            Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
        }
        
        // Now that the ontology is uploaded and the response included a valid response we
        // assume it will be able to be deleted
        
        // Get the resource and dump it to the console
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        deletionClientResource.getReference().addQueryParameter("deleteAllVersions", "true");
        
        RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null, MediaType.TEXT_PLAIN,
                Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadWithDeleteAllFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", "http://purl.org/obo/owl/test");
            deletionClientResource.getReference().addQueryParameter("deleteAllVersions", "true");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadWithOntologyUriFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", "http://purl.org/obo/owl/test");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadWithVersionUriFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", "http://purl.org/obo/owl/test");
            deletionClientResource.getReference().addQueryParameter("ontologyVersionUri",
                    "http://does.not.exist.example.org/test/delete/ontology");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     * @throws IOException
     * 
     */
    @Test
    public void testOntologyDeleteNoURIAfterUploadRdfXml2RdfXmlFalse() throws IOException
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoVersionRdfXml2RdfXmlFalse()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
        catch(final IOException ioe)
        {
            Assert.fail("Found unexpected IOException: " + ioe.getMessage());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     * @throws IOException
     * 
     */
    @Test
    public void testOntologyDeleteSingleVersionsRdfXml2RdfXmlFalse() throws IOException
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            deletionClientResource.getReference().addQueryParameter("ontologyVersionUri",
                    "http://does.not.exist.example.org/test/delete/ontology");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to delete a non-existent ontology version URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyDeleteSingleVersionsRdfXml2RdfXmlTrue() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        // final Representation results = creationClientResource.put(input,
        // MediaType.APPLICATION_RDF_XML);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
            
            final Ontology nextOntology = nextOntologies.get(0);
            
            Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
            Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
        }
        
        // Now that the ontology is uploaded and the response included a valid response we
        // assume it will be able to be deleted
        
        // Get the resource and dump it to the console
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        deletionClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        
        RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null, MediaType.TEXT_PLAIN,
                Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
    }
    
    /**
     * Tests the base ontology manager HTML page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerConnNegHtmlOnlyEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(1);
        connNegList.add(new Preference<MediaType>(MediaType.TEXT_HTML, 1.0f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting text/html MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        this.log.debug("resultString={}", resultString);
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString.contains("There are no managed ontologies matching your query."));
    }
    
    /**
     * Tests the base ontology manager HTML page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerConnNegHtmlRdfJsonOnlyEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(3);
        connNegList.add(new Preference<MediaType>(MediaType.TEXT_HTML, 1.0f));
        connNegList.add(new Preference<MediaType>(MediaType.APPLICATION_RDF_XML, 0.9f));
        connNegList.add(new Preference<MediaType>(MediaType.APPLICATION_JSON, 0.9f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting text/html MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString.contains("There are no managed ontologies matching your query."));
    }
    
    /**
     * Tests the base ontology manager JSON page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerConnNegJsonOnlyEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(1);
        connNegList.add(new Preference<MediaType>(RestletUtilMediaType.APPLICATION_RDF_JSON, 1.0f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests the base ontology manager JSON page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerConnNegJsonRdfHtmlOnlyEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(3);
        connNegList.add(new Preference<MediaType>(RestletUtilMediaType.APPLICATION_RDF_JSON, 1.0f));
        connNegList.add(new Preference<MediaType>(MediaType.APPLICATION_RDF_XML, 0.9f));
        connNegList.add(new Preference<MediaType>(MediaType.TEXT_HTML, 0.9f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting text/html MIME type for response
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests the base ontology manager RDF page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerConnNegRdfHtmlJsonOnlyEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(3);
        connNegList.add(new Preference<MediaType>(MediaType.APPLICATION_RDF_XML, 1.0f));
        connNegList.add(new Preference<MediaType>(MediaType.TEXT_HTML, 0.9f));
        connNegList.add(new Preference<MediaType>(MediaType.APPLICATION_JSON, 0.9f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting text/html MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests the base ontology manager RDF/XML page called with no parameters, and no ontologies
     * stored in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerConnNegRdfOnlyEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(1);
        connNegList.add(new Preference<MediaType>(MediaType.APPLICATION_RDF_XML, 1.0f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests the base ontology manager HTML page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerHtmlEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation results = creationClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting text/html MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString.contains("There are no managed ontologies matching your query."));
    }
    
    /**
     * Test the basic ontology manager HTML page after one ontology has been stored, with the
     * matching URI given as a parameter.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerHtmlOneOntologyMatchingUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        // final Representation results = creationClientResource.put(input,
        // MediaType.APPLICATION_RDF_XML);
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
            
            final Ontology nextOntology = nextOntologies.get(0);
            
            Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
            Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
        }
        
        // Get the resource and dump it to the console
        final ClientResource getClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        getClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        
        final Representation getResults = getClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), getResults.getMediaType().getName());
        
        final String resultString = IOUtils.toString(getResults.getStream());
        
        this.log.debug(resultString);
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString
                .contains("<table class=\"table table-striped table-hover\" id=\"ontologyList\">"));
        
        Assert.assertTrue(resultString.contains("<a class=\"ontology\""));
    }
    
    /**
     * Test the basic ontology manager HTML page after one ontology has been stored, with the
     * matching URI given as a parameter.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerHtmlOneOntologyNotMatchingUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        // final Representation results = creationClientResource.put(input,
        // MediaType.APPLICATION_RDF_XML);
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
            
            final Ontology nextOntology = nextOntologies.get(0);
            
            Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
            Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
        }
        
        // Get the resource and dump it to the console
        final ClientResource getClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        getClientResource.getReference().addQueryParameter("ontologyUri", "urn:ontologyresourceimpltest:notmatching");
        
        final Representation getResults = getClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), getResults.getMediaType().getName());
        
        final String resultString = IOUtils.toString(getResults.getStream());
        
        this.log.debug(resultString);
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString.contains("There are currently no managed ontologies."));
        
        Assert.assertFalse(resultString.contains("There are no managed ontologies matching your query."));
    }
    
    /**
     * Test the basic ontology manager HTML page after one ontology has been stored, but with no
     * URI, so it should instead list the ontology as one of the possible choices.
     * 
     * NOTE: This is not an integration test, as the HTML is not parsed by something like a browser.
     * See the HTMLUnit tests in oas-testwebsite for that test.
     * 
     * @throws Exception
     */
    @Test
    public void testOntologyManagerHtmlOneOntologyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
            
            final Ontology nextOntology = nextOntologies.get(0);
            
            Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
            Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
        }
        
        // Get the resource and dump it to the console
        final ClientResource getClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation getResults = getClientResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), getResults.getMediaType().getName());
        
        final String resultString = IOUtils.toString(getResults.getStream());
        
        this.log.debug(resultString);
        
        Assert.assertTrue(resultString.contains("<!-- OAS-testwebsite test Ontology Manager template -->"));
        
        Assert.assertTrue(resultString
                .contains("<table class=\"table table-striped table-hover\" id=\"ontologyList\">"));
        
        Assert.assertTrue(resultString.contains("<a class=\"ontology\""));
    }
    
    /**
     * Tests the base ontology manager JSON page called with no parameters, and no ontologies stored
     * in the database.
     * 
     * Basically should return 200, not an error. While not implemented it will return 501 and fail
     * the test.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerJsonEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation results = creationClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        this.log.debug("resultString=" + resultString);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests the base ontology manager RDF/XML page called with no parameters, and no ontologies
     * stored in the database.
     * 
     * Basically should return 200, not an error. While not implemented it will return 501 and fail
     * the test.
     * 
     * @throws Exception
     * 
     */
    @Test
    public void testOntologyManagerRdfxmlEmptyNoUri() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final Representation results = creationClientResource.get(MediaType.APPLICATION_RDF_XML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
}
