/**
 * OAS Webservice test implementations
 */
package com.github.ansell.oas.webservice.impl.test;

import java.io.InputStream;

import org.apache.commons.io.IOUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.helpers.StatementCollector;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * Unit test for OntologyResourceImpl.
 */
public class SearchOntologyLabelsResourceImplTest extends AbstractResourceImplTest
{
    private String ontologyUri;
    private String ontologyVersionUri;
    private String baseUri;
    
    @Override
    @Before
    public void setUp() throws Exception
    {
        super.setUp();
        
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        this.ontologyUri = "http://purl.org/obo/owl/test";
        this.ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        this.baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", this.ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", this.ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", this.baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        // final String rdfDocument =
        // TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri,
        // ontologyVersionUri);
        //
        // final Representation input =
        // new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML,
        // Language.DEFAULT,
        // CharacterSet.UTF_8);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        final String stringInput = IOUtils.toString(inputStream);
        
        // AbstractResourceImplTest.LOG.trace("input=");
        // AbstractResourceImplTest.LOG.trace(stringInput);
        
        // final Representation input = new InputRepresentation(inputStream,
        // MediaType.APPLICATION_RDF_XML);
        
        final Representation input = new StringRepresentation(stringInput, MediaType.APPLICATION_RDF_XML);
        
        // final Representation results = creationClientResource.put(input);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        // Expecting HTTP 201 Created response
        // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
    }
    
    /**
     * Test whether Ontologies can be searched using a JSON result type
     */
    @Test
    public void testOntologySearchJson() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource searchOntologyClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_SEARCH_ONTOLOGY_LABELS_PATH,
                        OasProps.DEF_SEARCH_ONTOLOGY_LABELS_PATH)));
        
        searchOntologyClientResource.getReference().addQueryParameter("searchterm", "plant");
        
        final Representation searchResults = searchOntologyClientResource.get(MediaType.APPLICATION_JSON);
        
        final StatementCollector results = new StatementCollector();
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        parser.setRDFHandler(results);
        
        parser.parse(searchResults.getStream(), "");
        
        Assert.assertEquals(2, results.getStatements().size());
        
    }
    
    /**
     * Test whether Ontologies can be searched using an RDF/XML result type.
     */
    @Test
    public void testOntologySearchRdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource searchOntologyClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_SEARCH_ONTOLOGY_LABELS_PATH,
                        OasProps.DEF_SEARCH_ONTOLOGY_LABELS_PATH)));
        
        searchOntologyClientResource.getReference().addQueryParameter("searchterm", "plant");
        
        final Representation searchResults = searchOntologyClientResource.get(MediaType.APPLICATION_RDF_XML);
        
        final StatementCollector results = new StatementCollector();
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        
        parser.setRDFHandler(results);
        
        parser.parse(searchResults.getStream(), "");
        
        // DebugUtils.printStatements(results.getStatements());
        
        Assert.assertEquals(2, results.getStatements().size());
        
    }
    
}
