/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import org.junit.Assert;
import org.junit.Test;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class WhoAmIResourceImplTest extends AbstractResourceImplTest
{
    
    @Test
    public void testGetWhoAmIAuthenticatedAdmin() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_WHOAMI_PATH,
                        OasProps.DEF_WHOAMI_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.TEXT_PLAIN, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final Representation representation = creationClientResource.get();
        
        final String result = representation.getText();
        this.assertFreemarker(result);
        
        // Success with user name
        Assert.assertEquals(RestletTestUtils.TEST_ADMIN_USERNAME, result);
        
    }
    
    @Test
    public void testGetWhoAmIAuthenticatedNonAdmin() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_WHOAMI_PATH,
                        OasProps.DEF_WHOAMI_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.TEXT_PLAIN, Status.SUCCESS_OK, this.testNoAdminPrivileges);
        
        final Representation representation = creationClientResource.get();
        
        final String result = representation.getText();
        this.assertFreemarker(result);
        
        // Success with user name
        Assert.assertEquals(RestletTestUtils.TEST_USERNAME, result);
    }
    
    @Test
    public void testGetWhoAmIUnauthenticated() throws Exception
    {
        
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_WHOAMI_PATH,
                        OasProps.DEF_WHOAMI_PATH)));
        try
        {
            creationClientResource.get();
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
}
